@@ -6,9 +6,7 @@ if [ -z "$ANDROID_ABI" ]; then
     echo "ANDROID_ABI not provided, building for ${ANDROID_ABI}"
 fi
 
-pushd ring-android
-./make-swig.sh
-popd
+TOP=$(pwd)/ring-android
 
 # Flags:
 
@@ -32,7 +30,6 @@ done
 export RELEASE
 export DAEMON_ONLY
 
-ANDROID_ABIS=""
 ANDROID_ABI_LIST="${ANDROID_ABI}"
 echo "Building ABIs: ${ANDROID_ABI_LIST}"
 for i in ${ANDROID_ABI_LIST}; do
@@ -42,7 +39,16 @@ for i in ${ANDROID_ABI_LIST}; do
     echo "$i build OK"
 done
 
+if [ -z "$RING_BUILD_FIREBASE" ]; then
+    echo "Building without Firebase support"
+else
+    GRADLE_PROPERTIES="-PbuildFirebase"
+    echo "Building with Firebase support"
+fi
 if [[ $DAEMON_ONLY -eq 0 ]]; then
-    export ANDROID_ABIS
-    make -b -j1 RELEASE=$RELEASE apk
+    if [[ $RELEASE -eq 1 ]]; then
+        cd $TOP && ./gradlew $GRADLE_PROPERTIES assembleRelease
+    else
+        cd $TOP && ./gradlew $GRADLE_PROPERTIES assembleDebug
+    fi
 fi
