@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
  *
  *  Author: Thibault Wittemberg <thibault.wittemberg@savoirfairelinux.com>
  *
@@ -49,6 +49,8 @@ import cx.ring.services.NotificationServiceImpl;
 import cx.ring.services.PreferencesService;
 import cx.ring.services.PresenceService;
 import cx.ring.services.SharedPreferencesServiceImpl;
+import cx.ring.services.VCardService;
+import cx.ring.services.VCardServiceImpl;
 import cx.ring.utils.Log;
 import dagger.Module;
 import dagger.Provides;
@@ -115,7 +117,7 @@ public class ServiceInjectionModule {
 
     @Provides
     @Singleton
-    CallService provideCallService(DaemonService daemonService) {
+    CallService provideCallService() {
         CallService callService = new CallService();
         mRingApplication.getRingInjectionComponent().inject(callService);
         return callService;
@@ -123,7 +125,7 @@ public class ServiceInjectionModule {
 
     @Provides
     @Singleton
-    ConferenceService provideConferenceService(DaemonService daemonService) {
+    ConferenceService provideConferenceService() {
         ConferenceService conferenceService = new ConferenceService();
         mRingApplication.getRingInjectionComponent().inject(conferenceService);
         return conferenceService;
@@ -131,7 +133,7 @@ public class ServiceInjectionModule {
 
     @Provides
     @Singleton
-    AccountService provideAccountService(DaemonService daemonService) {
+    AccountService provideAccountService() {
         AccountService accountService = new AccountService();
         mRingApplication.getRingInjectionComponent().inject(accountService);
         return accountService;
@@ -166,12 +168,19 @@ public class ServiceInjectionModule {
     ConversationFacade provideConversationFacade(
             ContactService contactService,
             HistoryService historyService,
-            CallService callService) {
-        ConversationFacade conversationFacade = new ConversationFacade(historyService, callService, contactService);
+            CallService callService,
+            AccountService accountService) {
+        ConversationFacade conversationFacade = new ConversationFacade(historyService, callService, contactService, accountService);
         mRingApplication.getRingInjectionComponent().inject(conversationFacade);
         return conversationFacade;
     }
 
+    @Provides
+    @Singleton
+    VCardService provideVCardService(Context context) {
+        return new VCardServiceImpl(context);
+    }
+
     @Provides
     @Named("DaemonExecutor")
     @Singleton
