@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *  Author: Alexandre Lision <alexandre.lision@savoirfairelinux.com>
@@ -15,7 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 package cx.ring.fragments;
 
@@ -24,11 +25,9 @@ import android.support.v7.preference.Preference;
 import android.support.v7.preference.PreferenceViewHolder;
 import android.support.v7.widget.RecyclerView;
 import android.util.AttributeSet;
-import android.util.Log;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
-import android.widget.AdapterView;
 import android.widget.BaseAdapter;
 import android.widget.CheckBox;
 import android.widget.LinearLayout;
@@ -40,22 +39,23 @@ import java.util.ArrayList;
 
 import cx.ring.R;
 import cx.ring.model.Codec;
-import cx.ring.views.dragsortlv.DragSortListView;
 
 class CodecPreference extends Preference {
     private static final String TAG = CodecPreference.class.getSimpleName();
 
-    private CodecAdapter listAdapter;
+    private final CodecAdapter listAdapter;
 
     public CodecPreference(Context context, AttributeSet attrs) {
-        this(context, attrs, 0);
+        super(context, attrs);
+        setWidgetLayoutResource(R.layout.frag_audio_mgmt);
+        listAdapter = new CodecAdapter(getContext());
+
     }
 
     public CodecPreference(Context context, AttributeSet attrs, int defStyleAttr) {
         super(context, attrs, defStyleAttr);
         setWidgetLayoutResource(R.layout.frag_audio_mgmt);
-        listAdapter = new CodecAdapter(context);
-        Log.w(TAG, "CodecPreference create");
+        listAdapter = new CodecAdapter(getContext());
     }
 
     private void setListViewHeight(ListView listView, LinearLayout llMain) {
@@ -84,35 +84,18 @@ class CodecPreference extends Preference {
     public void onBindViewHolder(PreferenceViewHolder holder) {
         super.onBindViewHolder(holder);
 
-        DragSortListView mCodecList = (DragSortListView) holder.findViewById(R.id.dndlistview);
-        if (mCodecList.getInputAdapter() != listAdapter)
+        ListView mCodecList = (ListView) holder.findViewById(R.id.dndlistview);
+        if (mCodecList.getAdapter() != listAdapter)
             mCodecList.setAdapter(listAdapter);
-        mCodecList.setDropListener(onDrop);
-        mCodecList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
-
-            @Override
-            public void onItemClick(AdapterView<?> arg0, View arg1, int pos, long arg3) {
-                listAdapter.getItem(pos).toggleState();
-                listAdapter.notifyDataSetChanged();
-                callChangeListener(getActiveCodecList());
-            }
+        mCodecList.setOnItemClickListener((arg0, arg1, pos, arg3) -> {
+            listAdapter.getItem(pos).toggleState();
+            listAdapter.notifyDataSetChanged();
+            callChangeListener(getActiveCodecList());
         });
 
         setListViewHeight(mCodecList, (LinearLayout) mCodecList.getParent());
     }
 
-    private DragSortListView.DropListener onDrop = new DragSortListView.DropListener() {
-        @Override
-        public void drop(int from, int to) {
-            if (from != to) {
-                Codec item = listAdapter.getItem(from);
-                listAdapter.remove(item);
-                listAdapter.insert(item, to);
-                callChangeListener(getActiveCodecList());
-            }
-        }
-    };
-
     ArrayList<Long> getActiveCodecList() {
         ArrayList<Long> results = new ArrayList<>();
         for (int i = 0; i < listAdapter.getCount(); ++i) {
@@ -127,26 +110,21 @@ class CodecPreference extends Preference {
         listAdapter.setDataset(codecs);
     }
 
+    void refresh() {
+        if (null != this.listAdapter) {
+            this.listAdapter.notifyDataSetChanged();
+        }
+    }
+
     private static class CodecAdapter extends BaseAdapter {
 
-        ArrayList<Codec> items;
+        private final ArrayList<Codec> items = new ArrayList<>();
         private Context mContext;
 
         CodecAdapter(Context context) {
-            items = new ArrayList<>();
             mContext = context;
         }
 
-        void insert(Codec item, int to) {
-            items.add(to, item);
-            notifyDataSetChanged();
-        }
-
-        void remove(Codec item) {
-            items.remove(item);
-            notifyDataSetChanged();
-        }
-
         @Override
         public int getCount() {
             return items.size();
@@ -177,9 +155,9 @@ class CodecPreference extends Preference {
                 rowView = inflater.inflate(R.layout.item_codec, parent, false);
 
                 entryView = new CodecView();
-                entryView.name = (TextView) rowView.findViewById(R.id.codec_name);
-                entryView.samplerate = (TextView) rowView.findViewById(R.id.codec_samplerate);
-                entryView.enabled = (CheckBox) rowView.findViewById(R.id.codec_checked);
+                entryView.name = rowView.findViewById(R.id.codec_name);
+                entryView.samplerate = rowView.findViewById(R.id.codec_samplerate);
+                entryView.enabled = rowView.findViewById(R.id.codec_checked);
                 rowView.setTag(entryView);
             } else {
                 entryView = (CodecView) rowView.getTag();
@@ -225,9 +203,8 @@ class CodecPreference extends Preference {
         }
 
         void setDataset(ArrayList<Codec> codecs) {
-            items = new ArrayList<>(codecs.size());
-            for (Codec c : codecs)
-                items.add(c);
+            items.clear();
+            items.addAll(codecs);
         }
 
         class CodecView {
@@ -236,10 +213,4 @@ class CodecPreference extends Preference {
             public CheckBox enabled;
         }
     }
-
-    void refresh() {
-        if (null != this.listAdapter) {
-            this.listAdapter.notifyDataSetChanged();
-        }
-    }
 }
