@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2017 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
  *
  *  Author: Hadrien Desousa <hadrien.desousa@savoirfairelinux.com>
  *
@@ -26,13 +26,21 @@ import android.net.NetworkInfo;
 public final class NetworkUtils {
     /**
      * Get the network info
-     *
-     * @param context
-     * @return
      */
     public static NetworkInfo getNetworkInfo(Context context) {
         ConnectivityManager connectivityManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
-        return connectivityManager.getActiveNetworkInfo();
+        if (connectivityManager != null) {
+            return connectivityManager.getActiveNetworkInfo();
+        }
+        return null;
     }
 
+    public static boolean isConnectivityAllowed(Context context, boolean allowMobile) {
+        NetworkInfo info = NetworkUtils.getNetworkInfo(context);
+        return info != null && info.isConnected() && (info.getType() != ConnectivityManager.TYPE_MOBILE || allowMobile);
+    }
+    public static boolean isPushAllowed(Context context, boolean allowMobile) {
+        NetworkInfo info = NetworkUtils.getNetworkInfo(context);
+        return info != null && (info.getType() != ConnectivityManager.TYPE_MOBILE || allowMobile);
+    }
 }
