@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
  *
  *  Author: Thibault Wittemberg <thibault.wittemberg@savoirfairelinux.com>
  *
@@ -19,14 +19,11 @@
  */
 package cx.ring.facades;
 
-import java.sql.SQLException;
+import java.io.File;
 import java.util.ArrayList;
-import java.util.Collections;
-import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.NavigableMap;
 import java.util.TreeMap;
 
 import javax.inject.Inject;
@@ -36,8 +33,9 @@ import cx.ring.model.CallContact;
 import cx.ring.model.Conference;
 import cx.ring.model.ConfigKey;
 import cx.ring.model.Conversation;
+import cx.ring.model.DataTransfer;
+import cx.ring.model.DataTransferEventCode;
 import cx.ring.model.HistoryCall;
-import cx.ring.model.HistoryEntry;
 import cx.ring.model.HistoryText;
 import cx.ring.model.SecureSipCall;
 import cx.ring.model.ServiceEvent;
@@ -52,6 +50,7 @@ import cx.ring.services.DeviceRuntimeService;
 import cx.ring.services.HardwareService;
 import cx.ring.services.HistoryService;
 import cx.ring.services.NotificationService;
+import cx.ring.utils.FileUtils;
 import cx.ring.utils.Log;
 import cx.ring.utils.Observable;
 import cx.ring.utils.Observer;
@@ -68,19 +67,18 @@ import cx.ring.utils.Tuple;
  */
 public class ConversationFacade extends Observable implements Observer<ServiceEvent> {
 
-    private final static String TAG = ConversationFacade.class.getName();
+    private final static String TAG = ConversationFacade.class.getSimpleName();
 
-    @Inject
-    AccountService mAccountService;
+    private final AccountService mAccountService;
 
-    private ContactService mContactService;
+    private final ContactService mContactService;
 
     @Inject
     ConferenceService mConferenceService;
 
-    private HistoryService mHistoryService;
+    private final HistoryService mHistoryService;
 
-    private CallService mCallService;
+    private final CallService mCallService;
 
     @Inject
     HardwareService mHardwareService;
@@ -91,16 +89,17 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
     @Inject
     DeviceRuntimeService mDeviceRuntimeService;
 
-    private Map<String, Conversation> mConversationMap;
+    private final Map<String, Conversation> mConversationMap = new HashMap<>();
 
-    public ConversationFacade(HistoryService historyService, CallService callService, ContactService contactService) {
-        mConversationMap = new HashMap<>();
+    public ConversationFacade(HistoryService historyService, CallService callService, ContactService contactService, AccountService accountService) {
         mHistoryService = historyService;
         mHistoryService.addObserver(this);
         mCallService = callService;
         mCallService.addObserver(this);
         mContactService = contactService;
         mContactService.addObserver(this);
+        mAccountService = accountService;
+        mAccountService.addObserver(this);
     }
 
     private Tuple<Conference, SipCall> getCall(String id) {
@@ -123,25 +122,19 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
         return mConversationMap;
     }
 
-    /**
-     * @param contact
-     * @return
-     */
     public Conversation getConversationByContact(CallContact contact) {
-        ArrayList<String> keys = contact.getIds();
-        for (String key : keys) {
-            Conversation conversation = mConversationMap.get(key);
-            if (conversation != null) {
-                return conversation;
+        if (contact != null) {
+            ArrayList<String> keys = contact.getIds();
+            for (String key : keys) {
+                Conversation conversation = mConversationMap.get(key);
+                if (conversation != null) {
+                    return conversation;
+                }
             }
         }
         return null;
     }
 
-    /**
-     * @param callId
-     * @return
-     */
     public Conversation getConversationByCallId(String callId) {
         for (Conversation conversation : mConversationMap.values()) {
             Conference conf = conversation.getConference(callId);
@@ -153,19 +146,21 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
     }
 
     /**
-     * @param contact
      * @return the started new conversation
      */
     public Conversation startConversation(CallContact contact) {
-        if (contact.isUnknown()) {
-            contact = mContactService.findContactByNumber(contact.getPhones().get(0).getNumber().getRawUriString());
-        }
         Conversation conversation = getConversationByContact(contact);
         if (conversation == null) {
             conversation = new Conversation(contact);
             mConversationMap.put(contact.getIds().get(0), conversation);
 
-            setConversationVisible();
+            Account account = mAccountService.getCurrentAccount();
+            if (account != null && account.isRing()) {
+                Uri number = contact.getPhones().get(0).getNumber();
+                if (number.isRingId()) {
+                    mAccountService.lookupAddress(account.getAccountID(), "", number.getRawRingId());
+                }
+            }
 
             setChanged();
             ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.CONVERSATIONS_CHANGED);
@@ -177,84 +172,12 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
     }
 
     /**
-     * @return the conversation local cache in a List
-     */
-    public ArrayList<Conversation> getConversationsList() {
-        ArrayList<Conversation> convs = new ArrayList<>(mConversationMap.values());
-        Collections.sort(convs, new Comparator<Conversation>() {
-            @Override
-            public int compare(Conversation lhs, Conversation rhs) {
-                return (int) ((rhs.getLastInteraction().getTime() - lhs.getLastInteraction().getTime()) / 1000l);
-            }
-        });
-        return convs;
-    }
-
-    /**
-     * @param id
      * @return the conversation from the local cache
      */
     public Conversation getConversationById(String id) {
         return mConversationMap.get(id);
     }
 
-    /**
-     * (also sends CONVERSATIONS_CHANGED event)
-     *
-     * @param newDisplayName
-     * @param ringId
-     */
-    public void updateConversationContactWithRingId(String newDisplayName, String ringId) {
-
-        if (newDisplayName == null || newDisplayName.isEmpty()) {
-            return;
-        }
-
-        Uri uri = new Uri(newDisplayName);
-        if (uri.isRingId()) {
-            return;
-        }
-
-        Conversation conversation = mConversationMap.get(CallContact.PREFIX_RING + ringId);
-        if (conversation == null) {
-            return;
-        }
-
-        CallContact contact = conversation.getContact();
-        if (contact == null || contact.getDisplayName().equals(newDisplayName)) {
-            return;
-        }
-
-        if (!contact.getDisplayName().contains(ringId)) {
-            contact.setUsername(newDisplayName);
-            return;
-        }
-
-        Uri ringIdUri = new Uri(ringId);
-        contact.getPhones().clear();
-        contact.getPhones().add(new cx.ring.model.Phone(ringIdUri, 0));
-        contact.setDisplayName(newDisplayName);
-        contact.setUsername(newDisplayName);
-
-        setChanged();
-        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.USERNAME_CHANGED);
-        notifyObservers(event);
-    }
-
-    public Conversation findOrStartConversationByNumber(Uri number) {
-        if (number == null || number.isEmpty()) {
-            return null;
-        }
-
-        for (Conversation conversation : mConversationMap.values()) {
-            if (conversation.getContact().hasNumber(number)) {
-                return conversation;
-            }
-        }
-
-        return startConversation(mContactService.findContactByNumber(number.getRawUriString()));
-    }
-
     public void sendTextMessage(String account, Uri to, String txt) {
         long id = mCallService.sendAccountTextMessage(account, to.getRawUriString(), txt);
         Log.i(TAG, "sendAccountTextMessage " + txt + " got id " + id);
@@ -272,66 +195,59 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
         mHistoryService.insertNewTextMessage(message);
     }
 
-    private void readTextMessage(TextMessage message) {
-        message.read();
-        HistoryText ht = new HistoryText(message);
-        mHistoryService.updateTextMessage(ht);
-    }
-
-    public void readConversation(Conversation conv) {
-        for (HistoryEntry h : conv.getRawHistory().values()) {
-            NavigableMap<Long, TextMessage> messages = h.getTextMessages();
-            for (TextMessage msg : messages.descendingMap().values()) {
-                if (msg.isRead()) {
-                    break;
-                }
-                readTextMessage(msg);
-            }
+    public void sendFile(String account, Uri to, File file) {
+        if (file == null) {
+            return;
         }
-        mNotificationService.cancelTextNotification(conv.getContact());
-        updateTextNotifications();
+
+        // check file
+        if (!file.exists()) {
+            Log.w(TAG, "sendFile: file not found");
+            return;
+        }
+
+        if (!file.canRead()) {
+            Log.w(TAG, "sendFile: file not readable");
+            return;
+        }
+        String peerId = to.getRawRingId();
+        DataTransfer transfer = new DataTransfer(0L, file.getName(), true, file.length(), 0, peerId, account);
+        // get generated ID
+        mHistoryService.insertDataTransfer(transfer);
+
+        File dest = mDeviceRuntimeService.getConversationPath(peerId, transfer.getStoragePath());
+        if (!FileUtils.moveFile(file, dest)) {
+            Log.e(TAG, "sendFile: can't move file to " + dest);
+            return;
+        }
+
+        // send file
+        mAccountService.sendFile(transfer, dest);
     }
 
     public void refreshConversations() {
         Log.d(TAG, "refreshConversations()");
-        try {
-            mHistoryService.getCallAndTextAsync();
-        } catch (SQLException e) {
-            Log.e(TAG, "unable to retrieve history calls and texts", e);
-        }
+        mHistoryService.getCallAndTextAsyncForAccount(mAccountService.getCurrentAccount().getAccountID());
     }
 
     public void updateTextNotifications() {
         Log.d(TAG, "updateTextNotifications()");
 
         for (Conversation conversation : mConversationMap.values()) {
-
-            if (conversation.isVisible()) {
-                mNotificationService.cancelTextNotification(conversation.getContact());
-                continue;
-            }
             TreeMap<Long, TextMessage> texts = conversation.getUnreadTextMessages();
-            if (texts.isEmpty() || texts.lastEntry().getValue().isNotified()) {
-                continue;
-            } else {
-                mNotificationService.cancelTextNotification(conversation.getContact());
-            }
 
+            if (texts.isEmpty() || conversation.isVisible()) {
+                mNotificationService.cancelTextNotification(conversation.getContact());
+                continue;
+            }
+            if (texts.lastEntry().getValue().isNotified()) {
+                continue;
+            }
             CallContact contact = conversation.getContact();
             mNotificationService.showTextNotification(contact, conversation, texts);
         }
     }
 
-    public synchronized Conference getConference(String id) {
-        for (Conversation conv : mConversationMap.values()) {
-            Conference conf = conv.getConference(id);
-            if (conf != null) {
-                return conf;
-            }
-        }
-        return null;
-    }
-
     public Conference getCurrentCallingConf() {
         for (Conversation c : getConversations().values()) {
             Conference conf = c.getCurrentCall();
@@ -342,34 +258,19 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
         return null;
     }
 
-    public void setConversationVisible() {
-        for (Conversation conv : mConversationMap.values()) {
-            boolean isConversationVisible = conv.isVisible();
-            String conversationKey = conv.getContact().getIds().get(0);
-            Conversation newConversation = mConversationMap.get(conversationKey);
-            if (newConversation != null) {
-                newConversation.setVisible(isConversationVisible);
-            }
-        }
-    }
-
-    public void removeConversation(String id) {
-        mConversationMap.remove(id);
-    }
-
-    private void parseNewMessage(TextMessage txt, String call) {
+    private void parseNewMessage(TextMessage txt) {
         Conversation conversation;
-        if (call != null && !call.isEmpty()) {
-            conversation = getConversationByCallId(call);
+        if (!StringUtils.isEmpty(txt.getCallId())) {
+            conversation = getConversationByCallId(txt.getCallId());
         } else {
             conversation = startConversation(mContactService.findContactByNumber(txt.getNumberUri().getRawUriString()));
             txt.setContact(conversation.getContact());
         }
-        if (conversation.isVisible()) {
-            txt.read();
-        }
 
         conversation.addTextMessage(txt);
+        if (txt.isRead()) {
+            mHistoryService.updateTextMessage(new HistoryText(txt));
+        }
     }
 
     private void parseHistoryCalls(List<HistoryCall> historyCalls, boolean acceptAllMessages) {
@@ -403,13 +304,26 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
         }
     }
 
+    private void parseHistoryTransfers(List<DataTransfer> historyTexts, boolean acceptAllMessages) {
+        for (DataTransfer htext : historyTexts) {
+            CallContact contact = mContactService.findContactByNumber(htext.getPeerId());
+            String key = contact.getIds().get(0);
+            if (mConversationMap.containsKey(key)) {
+                mConversationMap.get(key).addFileTransfer(htext);
+            } else if (acceptAllMessages) {
+                Conversation conversation = new Conversation(contact);
+                conversation.addFileTransfer(htext);
+                mConversationMap.put(key, conversation);
+            }
+        }
+    }
 
-    private void addContactDaemon(boolean acceptAllMessages) {
+    private void addContacts(boolean acceptAllMessages) {
         ArrayList<CallContact> contacts;
         if (acceptAllMessages) {
             contacts = new ArrayList<>(mContactService.getContactsNoBanned());
         } else {
-            contacts = new ArrayList<>(mContactService.getContactsConfirmed());
+            contacts = new ArrayList<>(mContactService.getContactsDaemon());
         }
         for (CallContact contact : contacts) {
             String key = contact.getIds().get(0);
@@ -472,33 +386,96 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
         }
     }
 
+    private void searchForRingIdInBlockchain() {
+        final String currentAccountId = mAccountService.getCurrentAccount().getAccountID();
+        for (Conversation conversation : mConversationMap.values()) {
+            CallContact contact = conversation.getContact();
+            if (contact == null) {
+                continue;
+            }
+            Uri contactUri = contact.getPhones().get(0).getNumber();
+            if (!contactUri.isRingId() || !StringUtils.isEmpty(contact.getUsername())) {
+                continue;
+            }
+            boolean currentAccountChecked = false;
+            for (String accountId : conversation.getAccountsUsed()) {
+                Account account = mAccountService.getAccount(accountId);
+                if (account == null || !account.isRing()) {
+                    continue;
+                }
+                if (accountId.equals(currentAccountId)) {
+                    currentAccountChecked = true;
+                }
+                mAccountService.lookupAddress(accountId, "", contactUri.getRawRingId());
+            }
+            if (!currentAccountChecked) {
+                mAccountService.lookupAddress(currentAccountId, "", contactUri.getRawRingId());
+            }
+        }
+    }
+
+    private void handleDataTransferEvent(DataTransfer transfer, DataTransferEventCode transferEventCode) {
+        CallContact contact = mContactService.findContactByNumber(transfer.getPeerId());
+        Conversation conversation = startConversation(contact);
+        if (transferEventCode == DataTransferEventCode.CREATED) {
+            if (transfer.isPicture() && !transfer.isOutgoing()) {
+                File path = mDeviceRuntimeService.getConversationPath(transfer.getPeerId(), transfer.getStoragePath());
+                mAccountService.acceptFileTransfer(transfer.getDataTransferId(), path.getAbsolutePath(), 0);
+            }
+            conversation.addFileTransfer(transfer);
+
+        } else {
+            conversation.updateFileTransfer(transfer, transferEventCode);
+        }
+        mNotificationService.showFileTransferNotification(transfer, transferEventCode, contact);
+    }
+
     @Override
     public void update(Observable observable, ServiceEvent event) {
+        if (event == null) {
+            return;
+        }
 
         ServiceEvent mEvent;
-        SipCall call;
-        String callId;
 
-        if (observable instanceof HistoryService && event != null) {
+        if (observable instanceof HistoryService) {
 
             switch (event.getEventType()) {
-                case INCOMING_MESSAGE:
+                case INCOMING_MESSAGE: {
                     TextMessage txt = event.getEventInput(ServiceEvent.EventInput.MESSAGE, TextMessage.class);
-                    callId = event.getEventInput(ServiceEvent.EventInput.CALL_ID, String.class);
+                    if (txt == null) {
+                        Log.d(TAG, "Received a null text message");
+                        return;
+                    }
 
-                    parseNewMessage(txt, callId);
+                    parseNewMessage(txt);
                     updateTextNotifications();
 
                     setChanged();
                     mEvent = new ServiceEvent(ServiceEvent.EventType.INCOMING_MESSAGE);
                     notifyObservers(mEvent);
                     break;
+                }
+                case ACCOUNT_MESSAGE_STATUS_CHANGED: {
+                    TextMessage newMsg = event.getEventInput(ServiceEvent.EventInput.MESSAGE, TextMessage.class);
+                    Conversation conv = getConversationByContact(mContactService.findContactByNumber(newMsg.getNumber()));
+                    if (conv != null) {
+                        conv.updateTextMessage(newMsg);
+                    }
+                    setChanged();
+
+                    mEvent = new ServiceEvent(ServiceEvent.EventType.INCOMING_MESSAGE);
+                    notifyObservers(mEvent);
+                    break;
+                }
                 case HISTORY_LOADED:
                     Account account = mAccountService.getCurrentAccount();
                     if (account != null) {
                         boolean acceptAllMessages = account.getDetailBoolean(ConfigKey.DHT_PUBLIC_IN);
 
-                        addContactDaemon(acceptAllMessages);
+                        mConversationMap.clear();
+
+                        addContacts(acceptAllMessages);
 
                         List<HistoryCall> historyCalls = (List<HistoryCall>) event.getEventInput(ServiceEvent.EventInput.HISTORY_CALLS, ArrayList.class);
                         parseHistoryCalls(historyCalls, acceptAllMessages);
@@ -506,36 +483,41 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
                         List<HistoryText> historyTexts = (List<HistoryText>) event.getEventInput(ServiceEvent.EventInput.HISTORY_TEXTS, ArrayList.class);
                         parseHistoryTexts(historyTexts, acceptAllMessages);
 
+                        List<DataTransfer> historyTransfers = (List<DataTransfer>) event.getEventInput(ServiceEvent.EventInput.HISTORY_TRANSFERS, ArrayList.class);
+                        parseHistoryTransfers(historyTransfers, acceptAllMessages);
+
                         aggregateHistory();
+
+                        searchForRingIdInBlockchain();
                     }
 
                     setChanged();
-                    mEvent = new ServiceEvent(ServiceEvent.EventType.HISTORY_LOADED);
+                    mEvent = new ServiceEvent(ServiceEvent.EventType.CONVERSATIONS_CHANGED);
                     notifyObservers(mEvent);
                     break;
                 case HISTORY_MODIFIED:
                     refreshConversations();
                     break;
             }
-        } else if (observable instanceof CallService && event != null) {
+        } else if (observable instanceof CallService) {
             Conversation conversation = null;
             Conference conference = null;
+            SipCall call;
             switch (event.getEventType()) {
                 case CALL_STATE_CHANGED:
-                    callId = event.getEventInput(ServiceEvent.EventInput.CALL_ID, String.class);
-                    int newState = SipCall.stateFromString(event.getEventInput(ServiceEvent.EventInput.STATE, String.class));
-                    call = mCallService.getCurrentCallForId(callId);
-
+                    call = event.getEventInput(ServiceEvent.EventInput.CALL, SipCall.class);
                     if (call == null) {
-                        Log.w(TAG, "CALL_STATE_CHANGED : call is null " + callId);
+                        Log.w(TAG, "CALL_STATE_CHANGED : call is null");
                         return;
                     }
+                    int newState = call.getCallState();
+                    mHardwareService.updateAudioState(call.isRinging() && call.isIncoming(), call.isOnGoing() && !call.isAudioOnly());
 
                     for (Conversation conv : mConversationMap.values()) {
-                        conference = conv.getConference(callId);
+                        conference = conv.getConference(call.getCallId());
                         if (conference != null) {
                             conversation = conv;
-                            Log.w(TAG, "CALL_STATE_CHANGED : found conversation " + callId);
+                            Log.w(TAG, "CALL_STATE_CHANGED : found conversation " + call.getCallId());
                             break;
                         }
                     }
@@ -546,21 +528,20 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
                         conversation.addConference(conference);
                     }
 
-                    conference.getParticipants().clear();
-                    conference.addParticipant(call);
-
                     Log.w(TAG, "CALL_STATE_CHANGED : updating call state to " + newState);
                     if ((call.isRinging() || newState == SipCall.State.CURRENT) && call.getTimestampStart() == 0) {
                         call.setTimestampStart(System.currentTimeMillis());
                     }
-
                     if ((newState == SipCall.State.CURRENT && call.isIncoming())
                             || newState == SipCall.State.RINGING && call.isOutGoing()) {
-                        mAccountService.sendProfile(callId, call.getAccount());
+                        mAccountService.sendProfile(call.getCallId(), call.getAccount());
                     } else if (newState == SipCall.State.HUNGUP
                             || newState == SipCall.State.BUSY
                             || newState == SipCall.State.FAILURE
                             || newState == SipCall.State.OVER) {
+                        mNotificationService.cancelCallNotification(call.getCallId().hashCode());
+                        mHardwareService.closeAudioState();
+
                         if (newState == SipCall.State.HUNGUP) {
                             call.setTimestampEnd(System.currentTimeMillis());
                         }
@@ -571,11 +552,10 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
                             call.setTimestampEnd(System.currentTimeMillis());
                         }
 
-                        mHistoryService.insertNewEntry(new Conference(call));
+                        mHistoryService.insertNewEntry(conference);
                         conference.removeParticipant(call);
                         conversation.addHistoryCall(new HistoryCall(call));
-                        mNotificationService.cancelCallNotification(call.getCallId().hashCode());
-                        mCallService.removeCallForId(callId);
+                        mCallService.removeCallForId(call.getCallId());
                     }
                     if (conference.getParticipants().isEmpty()) {
                         conversation.removeConference(conference);
@@ -584,12 +564,9 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
                     setChanged();
                     mEvent = new ServiceEvent(ServiceEvent.EventType.CALL_STATE_CHANGED);
                     notifyObservers(mEvent);
-
-                    refreshConversations();
                     break;
                 case INCOMING_CALL:
-                    callId = event.getEventInput(ServiceEvent.EventInput.CALL_ID, String.class);
-                    call = mCallService.getCurrentCallForId(callId);
+                    call = event.getEventInput(ServiceEvent.EventInput.CALL, SipCall.class);
                     conversation = startConversation(call.getContact());
                     conference = new Conference(call);
 
@@ -599,22 +576,56 @@ public class ConversationFacade extends Observable implements Observer<ServiceEv
                     mHardwareService.setPreviewSettings();
 
                     Conference currenConf = getCurrentCallingConf();
-                    mDeviceRuntimeService.updateAudioState(currenConf.isRinging()
-                            && currenConf.isIncoming());
+                    mHardwareService.updateAudioState(currenConf.isRinging()
+                            && currenConf.isIncoming(), false);
 
                     setChanged();
-                    ServiceEvent event1 = new ServiceEvent(ServiceEvent.EventType.INCOMING_CALL);
-                    notifyObservers(event1);
-
-                    refreshConversations();
+                    ServiceEvent serviceEvent = new ServiceEvent(ServiceEvent.EventType.INCOMING_CALL);
+                    notifyObservers(serviceEvent);
                     break;
             }
-        } else if (observable instanceof ContactService && event != null) {
+        } else if (observable instanceof ContactService) {
             switch (event.getEventType()) {
                 case CONTACTS_CHANGED:
                     refreshConversations();
                     break;
             }
+        } else if (observable instanceof AccountService) {
+            switch (event.getEventType()) {
+                case REGISTERED_NAME_FOUND: {
+                    int state = event.getEventInput(ServiceEvent.EventInput.STATE, Integer.class);
+                    if (state != 0) {
+                        break;
+                    }
+                    String accountId = event.getEventInput(ServiceEvent.EventInput.ACCOUNT_ID, String.class);
+                    String name = event.getEventInput(ServiceEvent.EventInput.NAME, String.class);
+                    Uri address = new Uri(event.getEventInput(ServiceEvent.EventInput.ADDRESS, String.class));
+                    if (mContactService.setRingContactName(accountId, address, name)) {
+                        setChanged();
+                        notifyObservers(new ServiceEvent(ServiceEvent.EventType.USERNAME_CHANGED));
+                    }
+                    break;
+                }
+                case DATA_TRANSFER: {
+                    DataTransferEventCode transferEventCode = event.getEventInput(ServiceEvent.EventInput.TRANSFER_EVENT_CODE, DataTransferEventCode.class);
+                    DataTransfer transfer = event.getEventInput(ServiceEvent.EventInput.TRANSFER_INFO, DataTransfer.class);
+                    handleDataTransferEvent(transfer, transferEventCode);
+                    setChanged();
+                    notifyObservers(new ServiceEvent(ServiceEvent.EventType.DATA_TRANSFER));
+                    break;
+                }
+            }
         }
     }
+
+    public SipCall placeCall(String accountId, String number, boolean video) {
+        String rawId = new Uri(number).getRawRingId();
+        Account account = mAccountService.getAccount(accountId);
+        if (account != null) {
+            CallContact contact = account.getContact(rawId);
+            if (contact == null)
+                mAccountService.addContact(accountId, rawId);
+        }
+        return mCallService.placeCall(rawId, number, video);
+    }
 }
\ No newline at end of file
