@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
  *
  *  Author: Thibault Wittemberg <thibault.wittemberg@savoirfairelinux.com>
  *
@@ -31,7 +31,7 @@ import javax.inject.Named;
 
 import cx.ring.daemon.Ringservice;
 import cx.ring.daemon.StringVect;
-import cx.ring.model.DaemonEvent;
+import cx.ring.model.ServiceEvent;
 import cx.ring.utils.FutureUtils;
 import cx.ring.utils.Log;
 import cx.ring.utils.Observable;
@@ -47,28 +47,15 @@ public class ConferenceService extends Observable {
     @Inject
     DeviceRuntimeService mDeviceRuntimeService;
 
-    private ConferenceCallbackHandler mCallbackHandler;
-
-    public ConferenceService() {
-        mCallbackHandler = new ConferenceCallbackHandler();
-    }
-
-    public ConferenceCallbackHandler getCallbackHandler() {
-        return mCallbackHandler;
-    }
-
     public void removeConference(final String confId) {
         FutureUtils.executeDaemonThreadCallable(
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "removeConference() thread running...");
-                        Ringservice.removeConference(confId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "removeConference() thread running...");
+                    Ringservice.removeConference(confId);
+                    return true;
                 }
         );
     }
@@ -78,14 +65,11 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "joinParticipant() thread running...");
-                        Ringservice.joinParticipant(selCallId, dragCallId);
-                        // Generate a CONF_CREATED callback
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "joinParticipant() thread running...");
+                    Ringservice.joinParticipant(selCallId, dragCallId);
+                    // Generate a CONF_CREATED callback
+                    return true;
                 }
         );
     }
@@ -95,13 +79,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "addParticipant() thread running...");
-                        Ringservice.addParticipant(callId, confId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "addParticipant() thread running...");
+                    Ringservice.addParticipant(callId, confId);
+                    return true;
                 }
         );
     }
@@ -111,13 +92,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "addMainParticipant() thread running...");
-                        Ringservice.addMainParticipant(confId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "addMainParticipant() thread running...");
+                    Ringservice.addMainParticipant(confId);
+                    return true;
                 }
         );
     }
@@ -127,13 +105,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "detachParticipant() thread running... " + callId);
-                        Ringservice.detachParticipant(callId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "detachParticipant() thread running... " + callId);
+                    Ringservice.detachParticipant(callId);
+                    return true;
                 }
         );
     }
@@ -143,13 +118,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "joinConference() thread running...");
-                        Ringservice.joinConference(selConfId, dragConfId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "joinConference() thread running...");
+                    Ringservice.joinConference(selConfId, dragConfId);
+                    return true;
                 }
         );
     }
@@ -159,13 +131,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "hangUpConference() thread running...");
-                        Ringservice.hangUpConference(confId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "hangUpConference() thread running...");
+                    Ringservice.hangUpConference(confId);
+                    return true;
                 }
         );
     }
@@ -175,13 +144,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "holdConference() thread running...");
-                        Ringservice.holdConference(confId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "holdConference() thread running...");
+                    Ringservice.holdConference(confId);
+                    return true;
                 }
         );
     }
@@ -191,13 +157,10 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "unholdConference() thread running...");
-                        Ringservice.unholdConference(confId);
-                        return true;
-                    }
+                () -> {
+                    Log.i(TAG, "unholdConference() thread running...");
+                    Ringservice.unholdConference(confId);
+                    return true;
                 }
         );
     }
@@ -208,12 +171,9 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 true,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "isConferenceParticipant() thread running...");
-                        return Ringservice.isConferenceParticipant(callId);
-                    }
+                () -> {
+                    Log.i(TAG, "isConferenceParticipant() thread running...");
+                    return Ringservice.isConferenceParticipant(callId);
                 }
         );
     }
@@ -223,15 +183,18 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 true,
-                new Callable<Map<String, ArrayList<String>>>() {
-                    @Override
-                    public Map<String, ArrayList<String>> call() throws Exception {
-                        Log.i(TAG, "getConferenceList() thread running...");
-                        StringVect callIds = Ringservice.getCallList();
-                        HashMap<String, ArrayList<String>> confs = new HashMap<>(callIds.size());
-                        for (int i = 0; i < callIds.size(); i++) {
-                            String callId = callIds.get(i);
-                            String confId = Ringservice.getConferenceId(callId);
+                (Callable<Map<String, ArrayList<String>>>) () -> {
+                    Log.i(TAG, "getConferenceList() thread running...");
+                    StringVect callIds = Ringservice.getCallList();
+                    HashMap<String, ArrayList<String>> confs = new HashMap<>(callIds.size());
+                    for (int i = 0; i < callIds.size(); i++) {
+                        String callId = callIds.get(i);
+                        String confId = Ringservice.getConferenceId(callId);
+
+                        Map<String, String> callDetails = Ringservice.getCallDetails(callId).toNative();
+
+                        //todo remove condition when callDetails does not contains sips ids anymore
+                        if (!callDetails.get("PEER_NUMBER").contains("sips")) {
                             if (confId == null || confId.isEmpty()) {
                                 confId = callId;
                             }
@@ -242,8 +205,8 @@ public class ConferenceService extends Observable {
                             }
                             calls.add(callId);
                         }
-                        return confs;
                     }
+                    return confs;
                 }
         );
     }
@@ -253,12 +216,9 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 true,
-                new Callable<List<String>>() {
-                    @Override
-                    public List<String> call() throws Exception {
-                        Log.i(TAG, "getParticipantList() thread running...");
-                        return new ArrayList<>(Ringservice.getParticipantList(confId));
-                    }
+                (Callable<List<String>>) () -> {
+                    Log.i(TAG, "getParticipantList() thread running...");
+                    return new ArrayList<>(Ringservice.getParticipantList(confId));
                 }
         );
     }
@@ -272,12 +232,9 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 true,
-                new Callable<String>() {
-                    @Override
-                    public String call() throws Exception {
-                        Log.i(TAG, "getConferenceDetails() thread running...");
-                        return Ringservice.getConferenceDetails(callId).get("CONF_STATE");
-                    }
+                () -> {
+                    Log.i(TAG, "getConferenceDetails() thread running...");
+                    return Ringservice.getConferenceDetails(callId).get("CONF_STATE");
                 }
         );
     }
@@ -287,42 +244,36 @@ public class ConferenceService extends Observable {
                 mExecutor,
                 mDeviceRuntimeService.provideDaemonThreadId(),
                 true,
-                new Callable<Map<String, String>>() {
-                    @Override
-                    public Map<String, String> call() throws Exception {
-                        Log.i(TAG, "getCredentials() thread running...");
-                        return Ringservice.getConferenceDetails(id).toNative();
-                    }
+                (Callable<Map<String, String>>) () -> {
+                    Log.i(TAG, "getCredentials() thread running...");
+                    return Ringservice.getConferenceDetails(id).toNative();
                 }
         );
     }
 
-    class ConferenceCallbackHandler {
+    void conferenceCreated(final String confId) {
+        Log.d(TAG, "conference created: " + confId);
+        setChanged();
+        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.CONFERENCE_CREATED);
+        event.addEventInput(ServiceEvent.EventInput.CONF_ID, confId);
+        notifyObservers(event);
+    }
 
-        void conferenceCreated(final String confId) {
-            Log.d(TAG, "conference created: " + confId);
-            setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.CONFERENCE_CREATED);
-            event.addEventInput(DaemonEvent.EventInput.CONF_ID, confId);
-            notifyObservers(event);
-        }
+    void conferenceRemoved(String confId) {
+        Log.d(TAG, "conference removed: " + confId);
+        setChanged();
+        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.CONFERENCE_REMOVED);
+        event.addEventInput(ServiceEvent.EventInput.CONF_ID, confId);
+        notifyObservers(event);
+    }
 
-        void conferenceRemoved(String confId) {
-            Log.d(TAG, "conference removed: " + confId);
-            setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.CONFERENCE_REMOVED);
-            event.addEventInput(DaemonEvent.EventInput.CONF_ID, confId);
-            notifyObservers(event);
-        }
-
-        void conferenceChanged(String confId, String state) {
-            Log.d(TAG, "conference changed: " + confId + ", " + state);
-            setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.CONFERENCE_CHANGED);
-            event.addEventInput(DaemonEvent.EventInput.CONF_ID, confId);
-            event.addEventInput(DaemonEvent.EventInput.STATE, state);
-            notifyObservers(event);
-        }
+    void conferenceChanged(String confId, String state) {
+        Log.d(TAG, "conference changed: " + confId + ", " + state);
+        setChanged();
+        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.CONFERENCE_CHANGED);
+        event.addEventInput(ServiceEvent.EventInput.CONF_ID, confId);
+        event.addEventInput(ServiceEvent.EventInput.STATE, state);
+        notifyObservers(event);
     }
 
 }
