@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2015-2018 Savoir-faire Linux Inc.
  *  Author: Stepan Salenikovich <stepan.salenikovich@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -42,8 +42,18 @@ struct _AccountGeneralTabPrivate
 {
     Account   *account;
     GtkWidget *grid_account;
+    GtkWidget* frame_parameters;
     GtkWidget *grid_parameters;
 
+    GtkWidget* entry_current_password;
+    GtkWidget* entry_new_password;
+    GtkWidget* entry_confirm_password;
+    GtkWidget* button_validate_password;
+    GtkWidget* label_error_change_passowrd;
+
+    GtkWidget* button_choose_file;
+    GtkWidget* label_export_informations;
+
     QMetaObject::Connection account_updated;
 };
 
@@ -77,7 +87,20 @@ account_general_tab_class_init(AccountGeneralTabClass *klass)
                                                 "/cx/ring/RingGnome/accountgeneraltab.ui");
 
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, grid_account);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, frame_parameters);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, grid_parameters);
+
+    /* change password view */
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, entry_current_password);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, entry_new_password);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, entry_confirm_password);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, button_validate_password);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, label_error_change_passowrd);
+
+    /* export account */
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, button_choose_file);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountGeneralTab, label_export_informations);
+
 }
 
 static void
@@ -90,14 +113,6 @@ account_alias_changed(GtkEditable *entry, AccountGeneralTab *view)
         priv->account->setDisplayName(gtk_entry_get_text(GTK_ENTRY(entry)));
 }
 
-static void
-entry_name_service_url_changed(GtkEditable *entry, AccountGeneralTab *view)
-{
-    g_return_if_fail(IS_ACCOUNT_GENERAL_TAB(view));
-    AccountGeneralTabPrivate *priv = ACCOUNT_GENERAL_TAB_GET_PRIVATE(view);
-    priv->account->setNameServiceURL(QString(gtk_editable_get_chars(entry, 0, -1)));
-}
-
 static void
 account_hostname_changed(GtkEditable *entry, AccountGeneralTab *view)
 {
@@ -145,33 +160,81 @@ account_mailbox_changed(GtkEditable *entry, AccountGeneralTab *view)
 }
 
 static void
-auto_answer(GtkToggleButton *checkbutton, AccountGeneralTab *view)
+reset_change_password(AccountGeneralTab *view)
 {
     g_return_if_fail(IS_ACCOUNT_GENERAL_TAB(view));
     AccountGeneralTabPrivate *priv = ACCOUNT_GENERAL_TAB_GET_PRIVATE(view);
-    priv->account->setAutoAnswer(gtk_toggle_button_get_active(checkbutton));
+    gtk_label_set_text(GTK_LABEL(priv->label_error_change_passowrd), "");
+    gtk_button_set_label(GTK_BUTTON(priv->button_validate_password), _("Change password"));
 }
 
-static void
-upnp_enabled(GtkToggleButton *checkbutton, AccountGeneralTab *view)
-{
-    g_return_if_fail(IS_ACCOUNT_GENERAL_TAB(view));
-    AccountGeneralTabPrivate *priv = ACCOUNT_GENERAL_TAB_GET_PRIVATE(view);
-    priv->account->setUpnpEnabled(gtk_toggle_button_get_active(checkbutton));
-}
 
 static void
-dtmf_set_rtp(GtkToggleButton *toggle_rtp, AccountGeneralTab *view)
+choose_export_file(AccountGeneralTab *view)
 {
     g_return_if_fail(IS_ACCOUNT_GENERAL_TAB(view));
     AccountGeneralTabPrivate *priv = ACCOUNT_GENERAL_TAB_GET_PRIVATE(view);
-    if (gtk_toggle_button_get_active(toggle_rtp)) {
-        g_debug("set DTMF over RTP");
-        priv->account->setDTMFType(DtmfType::OverRtp);
-    } else {
-        g_debug("set DTMF over SIP");
-        priv->account->setDTMFType(DtmfType::OverSip);
+    // Sauvegarder path
+    GtkWidget* dialog;
+    GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SAVE;
+    gint res;
+    gchar* filename = nullptr;
+
+    dialog = gtk_file_chooser_dialog_new(_("Save File"),
+                                         GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(view))),
+                                         action,
+                                         _("_Cancel"),
+                                         GTK_RESPONSE_CANCEL,
+                                         _("_Save"),
+                                         GTK_RESPONSE_ACCEPT,
+                                         nullptr);
+
+    std::string alias = priv->account->alias().toLocal8Bit().constData();
+    std::string uri = alias.empty()? "export.gz" : alias + ".gz";
+    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), uri.c_str());
+    res = gtk_dialog_run (GTK_DIALOG(dialog));
+
+    if (res == GTK_RESPONSE_ACCEPT) {
+        auto chooser = GTK_FILE_CHOOSER(dialog);
+        filename = gtk_file_chooser_get_filename(chooser);
     }
+    gtk_widget_destroy (dialog);
+
+    if (!filename) return;
+
+    // export account
+    priv->account->exportToFile(filename);
+    // Set informations label
+    std::string label = std::string(_("File exported to destination: ")) + std::string(filename);
+    gtk_label_set_text(GTK_LABEL(priv->label_export_informations), label.c_str());
+}
+
+static void
+change_password(AccountGeneralTab *view)
+{
+    g_return_if_fail(IS_ACCOUNT_GENERAL_TAB(view));
+    AccountGeneralTabPrivate *priv = ACCOUNT_GENERAL_TAB_GET_PRIVATE(view);
+
+    std::string current_password = gtk_entry_get_text(GTK_ENTRY(priv->entry_current_password));
+    std::string new_password = gtk_entry_get_text(GTK_ENTRY(priv->entry_new_password));
+    std::string confirm_password = gtk_entry_get_text(GTK_ENTRY(priv->entry_confirm_password));
+    if (new_password != confirm_password) {
+        gtk_label_set_text(GTK_LABEL(priv->label_error_change_passowrd), _("New password and its confirmation are different"));
+        return;
+    }
+    if (priv->account->changePassword(current_password.c_str(), new_password.c_str())) {
+        gtk_label_set_text(GTK_LABEL(priv->label_error_change_passowrd), "");
+        gtk_entry_set_text(GTK_ENTRY(priv->entry_current_password), "");
+        gtk_entry_set_text(GTK_ENTRY(priv->entry_new_password), "");
+        gtk_entry_set_text(GTK_ENTRY(priv->entry_confirm_password), "");
+        // NOTE: can't use account->archiveHasPassword here, because the account is not updated.
+        gtk_widget_set_visible(priv->entry_current_password, !new_password.empty());
+        gtk_button_set_label(GTK_BUTTON(priv->button_validate_password), _("Password changed!"));
+    } else {
+        gtk_button_set_label(GTK_BUTTON(priv->button_validate_password), _("Change password"));
+        gtk_label_set_text(GTK_LABEL(priv->label_error_change_passowrd), _("Incorrect password"));
+    }
+
 }
 
 static void
@@ -190,10 +253,6 @@ build_tab_view(AccountGeneralTab *view)
     GtkWidget *entry_password = NULL;
     GtkWidget *entry_proxy = NULL;
     GtkWidget *entry_voicemail = NULL;
-    GtkWidget *checkbutton_autoanswer = NULL;
-    GtkWidget *checkbutton_upnp = NULL;
-    GtkWidget *radiobutton_dtmf_rtp = NULL;
-    GtkWidget *radiobutton_dtmf_sip = NULL;
 
     /* build account grid */
 
@@ -321,63 +380,11 @@ build_tab_view(AccountGeneralTab *view)
         g_signal_connect(entry_voicemail, "changed", G_CALLBACK(account_mailbox_changed), view);
         gtk_grid_attach(GTK_GRID(priv->grid_parameters), entry_voicemail, 1, grid_row, 1, 1);
         ++grid_row;
+
     } else {
-        /* RING account */
-
-        /* Name service */
-        label = gtk_label_new(_("Name service URL"));
-        gtk_widget_set_halign(label, GTK_ALIGN_START);
-        gtk_grid_attach(GTK_GRID(priv->grid_parameters), label, 0, grid_row, 1, 1);
-        GtkWidget* entry_name_service_url = gtk_entry_new();
-        gtk_widget_set_halign(entry_name_service_url, GTK_ALIGN_START);
-        gtk_entry_set_text(GTK_ENTRY(entry_name_service_url), priv->account->nameServiceURL().toLocal8Bit().constData());
-        gtk_grid_attach(GTK_GRID(priv->grid_parameters), entry_name_service_url, 1, grid_row, 1, 1);
-        g_signal_connect(entry_name_service_url, "changed", G_CALLBACK(entry_name_service_url_changed), view);
-        ++grid_row;
-
+        gtk_widget_hide(priv->frame_parameters);
     }
-
-    /* auto answer */
-    checkbutton_autoanswer = gtk_check_button_new_with_label(_("Auto-answer calls"));
-    gtk_widget_set_halign(checkbutton_autoanswer, GTK_ALIGN_START);
-    gtk_grid_attach(GTK_GRID(priv->grid_parameters), checkbutton_autoanswer, 0, grid_row, 1, 1);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_autoanswer), priv->account->isAutoAnswer());
-    g_signal_connect(checkbutton_autoanswer, "toggled", G_CALLBACK(auto_answer), view);
-    ++grid_row;
-
-    /* upnp */
-    checkbutton_upnp = gtk_check_button_new_with_label(_("UPnP enabled"));
-    gtk_widget_set_halign(checkbutton_upnp, GTK_ALIGN_START);
-    gtk_grid_attach(GTK_GRID(priv->grid_parameters), checkbutton_upnp, 0, grid_row, 1, 1);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_upnp), priv->account->isUpnpEnabled());
-    g_signal_connect(checkbutton_upnp, "toggled", G_CALLBACK(upnp_enabled), view);
-    ++grid_row;
-
-    /* DTMF tone type */
-    label = gtk_label_new(_("DTMF tone type:"));
-    gtk_widget_set_halign(label, GTK_ALIGN_START);
-    gtk_grid_attach(GTK_GRID(priv->grid_parameters), label, 0, grid_row, 1, 1);
-
-    GtkWidget *dtmf_box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(dtmf_box), GTK_BUTTONBOX_START);
-    gtk_box_set_spacing(GTK_BOX(dtmf_box), 10);
-    radiobutton_dtmf_rtp = gtk_radio_button_new_with_label(NULL, "RTP");
-    radiobutton_dtmf_sip = gtk_radio_button_new_with_label_from_widget(
-        GTK_RADIO_BUTTON(radiobutton_dtmf_rtp),
-        "SIP");
-    gtk_toggle_button_set_active(
-        priv->account->DTMFType() == DtmfType::OverRtp ?
-            GTK_TOGGLE_BUTTON(radiobutton_dtmf_rtp) : GTK_TOGGLE_BUTTON(radiobutton_dtmf_sip),
-        TRUE);
-    g_signal_connect(radiobutton_dtmf_rtp, "toggled", G_CALLBACK(dtmf_set_rtp), view);
-    gtk_box_pack_start(GTK_BOX(dtmf_box),
-                       radiobutton_dtmf_rtp,
-                       FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(dtmf_box),
-                       radiobutton_dtmf_sip,
-                       FALSE, TRUE, 0);
-    gtk_grid_attach(GTK_GRID(priv->grid_parameters), dtmf_box, 1, grid_row, 1, 1);
-    ++grid_row;
+    gtk_widget_show_all(priv->grid_parameters);
 
     /* update account parameters if model is updated */
     priv->account_updated = QObject::connect(
@@ -393,17 +400,15 @@ build_tab_view(AccountGeneralTab *view)
                 gtk_entry_set_text(GTK_ENTRY(entry_proxy), priv->account->proxy().toLocal8Bit().constData());
                 gtk_entry_set_text(GTK_ENTRY(entry_voicemail), priv->account->mailbox().toLocal8Bit().constData());
             }
-
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_autoanswer), priv->account->isAutoAnswer());
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_upnp), priv->account->isUpnpEnabled());
-            gtk_toggle_button_set_active(
-                priv->account->DTMFType() == DtmfType::OverRtp ?
-                    GTK_TOGGLE_BUTTON(radiobutton_dtmf_rtp) : GTK_TOGGLE_BUTTON(radiobutton_dtmf_sip),
-                    TRUE);
         }
     );
 
-    gtk_widget_show_all(priv->grid_parameters);
+    g_signal_connect_swapped(priv->button_validate_password, "clicked", G_CALLBACK(change_password), view);
+    g_signal_connect_swapped(priv->entry_current_password, "changed", G_CALLBACK(reset_change_password), view);
+    g_signal_connect_swapped(priv->entry_new_password, "changed", G_CALLBACK(reset_change_password), view);
+    g_signal_connect_swapped(priv->entry_confirm_password, "changed", G_CALLBACK(reset_change_password), view);
+    g_signal_connect_swapped(priv->button_choose_file, "clicked", G_CALLBACK(choose_export_file), view);
+    gtk_widget_set_visible(priv->entry_current_password, priv->account->archiveHasPassword());
 
 }
 
