@@ -1,8 +1,9 @@
 /*
- *  Copyright (C) 2014-2017 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2018 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *  Author: Simon Désaulniers <simon.desaulniers@gmail.com>
+ *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,11 +27,13 @@
 #endif
 
 #include "security/tls_session.h"
+#include "security/diffie-hellman.h"
 #include "sip/sipaccountbase.h"
 
 #include "noncopyable.h"
 #include "ip_utils.h"
 #include "ring_types.h" // enable_if_base_of
+#include "security/certstore.h"
 
 #include <opendht/dhtrunner.h>
 #include <opendht/default_types.h>
@@ -67,33 +70,21 @@ namespace dev
 
 namespace ring {
 
-namespace Conf {
-constexpr const char* const DHT_PORT_KEY = "dhtPort";
-constexpr const char* const DHT_VALUES_PATH_KEY = "dhtValuesPath";
-constexpr const char* const DHT_CONTACTS = "dhtContacts";
-constexpr const char* const DHT_PUBLIC_PROFILE = "dhtPublicProfile";
-constexpr const char* const DHT_PUBLIC_IN_CALLS = "dhtPublicInCalls";
-constexpr const char* const DHT_ALLOW_PEERS_FROM_HISTORY = "allowPeersFromHistory";
-constexpr const char* const DHT_ALLOW_PEERS_FROM_CONTACT = "allowPeersFromContact";
-constexpr const char* const DHT_ALLOW_PEERS_FROM_TRUSTED = "allowPeersFromTrusted";
-constexpr const char* const ETH_KEY = "ethKey";
-constexpr const char* const ETH_PATH = "ethPath";
-constexpr const char* const ETH_ACCOUNT = "ethAccount";
-constexpr const char* const RING_CA_KEY = "ringCaKey";
-constexpr const char* const RING_ACCOUNT_KEY = "ringAccountKey";
-constexpr const char* const RING_ACCOUNT_CERT = "ringAccountCert";
-constexpr const char* const RING_ACCOUNT_RECEIPT = "ringAccountReceipt";
-constexpr const char* const RING_ACCOUNT_RECEIPT_SIG = "ringAccountReceiptSignature";
-constexpr const char* const RING_ACCOUNT_CRL = "ringAccountCRL";
-}
-
 class IceTransport;
+struct Contact;
+struct AccountArchive;
+class DhtPeerConnector;
+class PeerConnection;
 
 class RingAccount : public SIPAccountBase {
+    private:
+        struct PeerConnectionMsg;
+
     public:
         constexpr static const char* const ACCOUNT_TYPE = "RING";
         constexpr static const in_port_t DHT_DEFAULT_PORT = 4222;
         constexpr static const char* const DHT_DEFAULT_BOOTSTRAP = "bootstrap.ring.cx";
+        constexpr static const char* const DHT_DEFAULT_PROXY = "dhtproxy.ring.cx";
         constexpr static const char* const DHT_TYPE_NS = "cx.ring";
 
         /* constexpr */ static const std::pair<uint16_t, uint16_t> DHT_PORT_RANGE;
@@ -232,7 +223,8 @@ class RingAccount : public SIPAccountBase {
          * Implementation of Account::newOutgoingCall()
          * Note: keep declaration before newOutgoingCall template.
          */
-        std::shared_ptr<Call> newOutgoingCall(const std::string& toUrl) override;
+        std::shared_ptr<Call> newOutgoingCall(const std::string& toUrl,
+                                              const std::map<std::string, std::string>& volatileCallDetails = {}) override;
 
         /**
          * Create outgoing SIPCall.
@@ -244,22 +236,23 @@ class RingAccount : public SIPAccountBase {
 #ifndef RING_UWP
         template <class T=SIPCall>
         std::shared_ptr<enable_if_base_of<T, SIPCall> >
-        newOutgoingCall(const std::string& toUrl);
+        newOutgoingCall(const std::string& toUrl, const std::map<std::string, std::string>& volatileCallDetails = {});
 #else
         template <class T>
         std::shared_ptr<T>
-        newOutgoingCall(const std::string& toUrl);
+        newOutgoingCall(const std::string& toUrl, const std::map<std::string, std::string>& volatileCallDetails = {});
 #endif
 
         /**
          * Create incoming SIPCall.
          * @param[in] from The origin of the call
+         * @param details use to set some specific details
          * @return std::shared_ptr<T> A shared pointer on the created call.
          *      The type of this instance is given in template argument.
          *      This type can be any base class of SIPCall class (included).
          */
         virtual std::shared_ptr<SIPCall>
-        newIncomingCall(const std::string& from = {}) override;
+        newIncomingCall(const std::string& from, const std::map<std::string, std::string>& details = {}) override;
 
         virtual bool isTlsEnabled() const override {
             return true;
@@ -278,7 +271,6 @@ class RingAccount : public SIPAccountBase {
         }
 
         bool setCertificateStatus(const std::string& cert_id, tls::TrustStore::PermissionStatus status);
-        bool setCertificateStatus(const std::string& cert_id, tls::TrustStatus status);
 
         std::vector<std::string> getCertificatesByStatus(tls::TrustStore::PermissionStatus status);
 
@@ -286,19 +278,35 @@ class RingAccount : public SIPAccountBase {
         bool findCertificate(const dht::InfoHash& h, std::function<void(const std::shared_ptr<dht::crypto::Certificate>&)>&& cb = {});
 
         /* contact requests */
-        std::map<std::string, std::string> getTrustRequests() const;
+        std::vector<std::map<std::string, std::string>> getTrustRequests() const;
         bool acceptTrustRequest(const std::string& from);
         bool discardTrustRequest(const std::string& from);
 
+        /**
+         * Add contact to the account contact list.
+         * Set confirmed if we know the contact also added us.
+         */
+        void addContact(const std::string& uri, bool confirmed = false);
+        void removeContact(const std::string& uri, bool banned = true);
+        std::vector<std::map<std::string, std::string>> getContacts() const;
+
+        ///
+        /// Obtain details about one account contact in serializable form.
+        ///
+        std::map<std::string, std::string> getContactDetails(const std::string& uri) const;
+
         void sendTrustRequest(const std::string& to, const std::vector<uint8_t>& payload);
+        void sendTrustRequestConfirm(const dht::InfoHash& to);
         virtual void sendTextMessage(const std::string& to, const std::map<std::string, std::string>& payloads, uint64_t id) override;
 
+        /* Devices */
         void addDevice(const std::string& password);
-
+        bool exportArchive(const std::string& destinationPath);
         bool revokeDevice(const std::string& password, const std::string& device);
-
         std::map<std::string, std::string> getKnownDevices() const;
 
+        bool changeArchivePassword(const std::string& password_old, const std::string& password_new);
+
         void connectivityChanged() override;
 
         // overloaded methods
@@ -310,18 +318,66 @@ class RingAccount : public SIPAccountBase {
         void registerName(const std::string& password, const std::string& name);
 #endif
 
+        ///
+        /// Send a E2E connection request to a given peer
+        ///
+        /// /// \param[in] peer_id RingID on request's recipiant
+        ///
+        void requestPeerConnection(const std::string& peer,
+                                   std::function<void(PeerConnection*)> connect_cb);
+
+        std::vector<std::string> publicAddresses();
+
+
+        /// \return true if the given DHT message identifier has been treated
+        /// \note if message has not been treated yet this method store this id and returns true at further calls
+        bool isMessageTreated(unsigned int id) ;
+
+        dht::DhtRunner& dht() { return dht_; }
+
+        const dht::crypto::Identity& identity() const { return identity_; }
+
+        const std::shared_future<tls::DhParams> dhParams() const { return dhParams_; }
+
+        void forEachDevice(const dht::InfoHash& to,
+                           std::function<void(const std::shared_ptr<RingAccount>&,
+                                              const dht::InfoHash&)> op,
+                           std::function<void(bool)> end = {});
+
+        /**
+         * Inform that a potential peer device have been found.
+         * Returns true only if the device certificate is a valid Ring device certificate.
+         * In that case (true is returned) the account_id parameter is set to the peer account ID.
+         */
+        static bool foundPeerDevice(const std::shared_ptr<dht::crypto::Certificate>& crt, dht::InfoHash& account_id);
+
+        /**
+         * Start or stop to use the proxy client
+         * @param address of the proxy
+         * @param deviceKey the device key for push notifications (empty to not use it)
+         */
+        void enableProxyClient(bool enable);
+
+        void setPushNotificationToken(const std::string& pushDeviceToken = "");
+
+        /**
+         * To be called by clients with relevent data when a push notification is received.
+         */
+        void pushNotificationReceived(const std::string& from, const std::map<std::string, std::string>& data);
+
     private:
         NON_COPYABLE(RingAccount);
 
+        using clock = std::chrono::system_clock;
+        using time_point = clock::time_point;
+
         /**
          * Private structures
          */
         struct PendingCall;
         struct PendingMessage;
-        struct SavedTrustRequest;
         struct TrustRequest;
         struct KnownDevice;
-        struct ArchiveContent;
         struct DeviceAnnouncement;
         struct DeviceSync;
         struct BuddyInfo;
@@ -356,15 +412,15 @@ class RingAccount : public SIPAccountBase {
         void onTrackedBuddyOnline(std::map<dht::InfoHash, BuddyInfo>::iterator& buddy_info_it, const dht::InfoHash& device_id);
 
         void doRegister_();
-        void incomingCall(dht::IceCandidates&& msg, const std::shared_ptr<dht::crypto::Certificate>& from);
+        void incomingCall(dht::IceCandidates&& msg, const std::shared_ptr<dht::crypto::Certificate>& from_cert, const dht::InfoHash& from);
 
         const dht::ValueType USER_PROFILE_TYPE = {9, "User profile", std::chrono::hours(24 * 7)};
 
         void handleEvents();
 
-        void forEachDevice(const dht::InfoHash& to, std::function<void(const std::shared_ptr<RingAccount>&, const dht::InfoHash&)> op, std::function<void(bool)> end = {});
+        void startOutgoingCall(const std::shared_ptr<SIPCall>& call, const std::string toUri);
 
-        void createOutgoingCall(const std::shared_ptr<SIPCall>& call, const std::string& to_id, IpAddr target);
+        void onConnectedOutgoingCall(SIPCall& call, const std::string& to_id, IpAddr target);
 
         /**
          * Set the internal state for this account, mainly used to manage account details from the client application.
@@ -372,27 +428,28 @@ class RingAccount : public SIPAccountBase {
          */
         virtual void setAccountDetails(const std::map<std::string, std::string> &details) override;
 
-        void startOutgoingCall(const std::shared_ptr<SIPCall>& call, const std::string toUri);
-
         /**
          * Start a SIP Call
          * @param call  The current call
          * @return true if all is correct
          */
-        bool SIPStartCall(const std::shared_ptr<SIPCall>& call, IpAddr target);
+        bool SIPStartCall(SIPCall& call, IpAddr target);
 
         /**
          * Inform that a potential account device have been found.
          * Returns true if the device have been validated to be part of this account
          */
-        bool foundAccountDevice(const std::shared_ptr<dht::crypto::Certificate>& crt, const std::string& name = {});
+        bool foundAccountDevice(const std::shared_ptr<dht::crypto::Certificate>& crt, const std::string& name = {}, const time_point& last_sync = time_point::min());
 
         /**
-         * Inform that a potential peer device have been found.
-         * Returns true only if the device certificate is a valid Ring device certificate.
-         * In that case (true is returned) the account_id parameter is set to the peer account ID.
+         * For a call with (from_device, from_account), check the peer certificate chain (cert_list, cert_num)
+         * with session check status.
+         * Put deserialized certificate to cert_out;
          */
-        bool foundPeerDevice(const std::shared_ptr<dht::crypto::Certificate>& crt, dht::InfoHash& account_id);
+        static pj_status_t checkPeerTlsCertificate(dht::InfoHash from_device, dht::InfoHash from_account,
+                                unsigned status,
+                                const gnutls_datum_t* cert_list, unsigned cert_num,
+                                std::shared_ptr<dht::crypto::Certificate>& cert_out);
 
         /**
          * Check that a peer is authorised to talk to us.
@@ -402,6 +459,8 @@ class RingAccount : public SIPAccountBase {
          */
         void onPeerMessage(const dht::InfoHash& peer_device, std::function<void(const std::shared_ptr<dht::crypto::Certificate>& crt, const dht::InfoHash& account_id)>);
 
+        void onTrustRequest(const dht::InfoHash& peer_account, const dht::InfoHash& peer_device, time_t received , bool confirm, std::vector<uint8_t>&& payload);
+
         /**
          * Maps require port via UPnP
          */
@@ -442,6 +501,7 @@ class RingAccount : public SIPAccountBase {
         std::string ethAccount_ {};
 
         std::string archivePath_ {};
+        bool archiveHasPassword_ {true};
 
         std::string receipt_ {};
         std::vector<uint8_t> receiptSignature_ {};
@@ -449,8 +509,16 @@ class RingAccount : public SIPAccountBase {
 
         std::map<dht::InfoHash, TrustRequest> trustRequests_;
         void loadTrustRequests();
-        void saveTrustRequests();
+        void saveTrustRequests() const;
 
+        std::map<dht::InfoHash, Contact> contacts_;
+        void loadContacts();
+        void saveContacts() const;
+        void updateContact(const dht::InfoHash&, const Contact&);
+
+        // Trust store with Ring account main certificate as the only CA
+        dht::crypto::TrustList accountTrust_;
+        // Trust store for to match peer certificates
         tls::TrustStore trust_;
 
         std::shared_ptr<dht::Value> announce_;
@@ -462,28 +530,29 @@ class RingAccount : public SIPAccountBase {
         std::recursive_mutex buddyInfoMtx;
         std::map<dht::InfoHash, BuddyInfo> trackedBuddies_;
 
-        void loadAccount(const std::string& archive_password = {}, const std::string& archive_pin = {});
+        void loadAccount(const std::string& archive_password = {}, const std::string& archive_pin = {}, const std::string& archive_path = {});
+        void loadAccountFromFile(const std::string& archive_path, const std::string& archive_password);
         void loadAccountFromDHT(const std::string& archive_password, const std::string& archive_pin);
+        void loadAccountFromArchive(AccountArchive&& archive, const std::string& archive_password);
 
         bool hasCertificate() const;
         bool hasPrivateKey() const;
-        bool hasSignedReceipt();
-        bool needsMigration() const;
+        bool useIdentity(const dht::crypto::Identity& id);
+        static bool needsMigration(const dht::crypto::Identity& id);
 
         std::string makeReceipt(const dht::crypto::Identity& id);
         void createRingDevice(const dht::crypto::Identity& id);
-        void initRingDevice(const ArchiveContent& a);
-        bool migrateAccount(const std::string& pwd);
-        static bool updateCertificates(ArchiveContent& archive, dht::crypto::Identity& device);
+        void initRingDevice(const AccountArchive& a);
+        void migrateAccount(const std::string& pwd, dht::crypto::Identity& device);
+        static bool updateCertificates(AccountArchive& archive, dht::crypto::Identity& device);
 
-        void createAccount(const std::string& archive_password);
-        std::vector<uint8_t> makeArchive(const ArchiveContent& content) const;
-        void saveArchive(const ArchiveContent& content, const std::string& pwd);
-        ArchiveContent readArchive(const std::string& pwd) const;
-        static ArchiveContent loadArchive(const std::vector<uint8_t>& data);
-        std::vector<std::pair<sockaddr_storage, socklen_t>> loadBootstrap() const;
+        void createAccount(const std::string& archive_password, dht::crypto::Identity&& migrate);
+        void updateArchive(AccountArchive& content) const;
+        void saveArchive(AccountArchive& content, const std::string& pwd);
+        AccountArchive readArchive(const std::string& pwd) const;
+        std::vector<dht::SockAddr> loadBootstrap() const;
 
-        std::pair<std::string, std::string> saveIdentity(const dht::crypto::Identity id, const std::string& path) const;
+        static std::pair<std::string, std::string> saveIdentity(const dht::crypto::Identity id, const std::string& path, const std::string& name);
         void saveNodes(const std::vector<dht::NodeExport>&) const;
         void saveValues(const std::vector<dht::ValuesExport>&) const;
 
@@ -493,14 +562,14 @@ class RingAccount : public SIPAccountBase {
         void loadTreatedMessages();
         void saveTreatedMessages() const;
 
-        void loadKnownDevicesOld();
         void loadKnownDevices();
         void saveKnownDevices() const;
 
         void replyToIncomingIceMsg(const std::shared_ptr<SIPCall>&,
                                    const std::shared_ptr<IceTransport>&,
                                    const dht::IceCandidates&,
-                                   const std::shared_ptr<dht::crypto::Certificate>&);
+                                   const std::shared_ptr<dht::crypto::Certificate>& from_cert,
+                                   const dht::InfoHash& from);
 
         static tls::DhParams loadDhParams(const std::string path);
 
@@ -509,9 +578,10 @@ class RingAccount : public SIPAccountBase {
          * and certPath_ a valid certificate file, load and returns them.
          * Otherwise, generate a new identity and returns it.
          */
-        dht::crypto::Identity loadIdentity();
+        dht::crypto::Identity loadIdentity(const std::string& crt_path, const std::string& key_path, const std::string& key_pwd) const;
         std::vector<dht::NodeExport> loadNodes() const;
         std::vector<dht::ValuesExport> loadValues() const;
+        mutable std::mutex dhtValuesMtx_;
 
         bool dhtPublicInCalls_ {true};
 
@@ -528,6 +598,13 @@ class RingAccount : public SIPAccountBase {
          */
         UsedPort dhtPortUsed_ {};
 
+        /**
+         * Proxy
+         */
+        bool proxyEnabled_;
+        std::string proxyServer_;
+        std::string deviceKey_;
+
         /**
          * The TLS settings, used only if tls is chosen as a sip transport.
          */
@@ -564,6 +641,14 @@ class RingAccount : public SIPAccountBase {
         std::shared_ptr<IceTransport> createIceTransport(const Args&... args);
 
         void registerDhtAddress(IceTransport&);
+
+        std::unique_ptr<DhtPeerConnector> dhtPeerConnector_;
 };
 
+static inline std::ostream& operator<< (std::ostream& os, const RingAccount& acc)
+{
+    os << "[Account " << acc.getAccountID() << "] ";
+    return os;
+}
+
 } // namespace ring
